# Generated by Django 2.2.6 on 2019-10-03 15:05

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('ansible', '0012_auto_20190906_2253'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='collectionimport',
            options={'ordering': ['task__pulp_created']},
        ),
        migrations.RenameField(
            model_name='collection',
            old_name='_created',
            new_name='pulp_created',
        ),
        migrations.RenameField(
            model_name='collection',
            old_name='_id',
            new_name='pulp_id',
        ),
        migrations.RenameField(
            model_name='collection',
            old_name='_last_updated',
            new_name='pulp_last_updated',
        ),
        migrations.RenameField(
            model_name='tag',
            old_name='_created',
            new_name='pulp_created',
        ),
        migrations.RenameField(
            model_name='tag',
            old_name='_last_updated',
            new_name='pulp_last_updated',
        ),
        migrations.AlterField(
            model_name='collectionversion',
            name='collection',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='versions', to='ansible.Collection', to_field='pulp_id'),
        ),
    ]
