# Generated by Django 2.2.3 on 2019-08-02 12:00

import django.contrib.postgres.fields
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('ansible', '0002_advanced_collections'),
    ]

    operations = [
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('_id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('_created', models.DateTimeField(auto_now_add=True)),
                ('_last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('name', models.CharField(editable=False, max_length=32, unique=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='collectionversion',
            name='authors',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=64), default=list, editable=False, size=None),
        ),
        migrations.AddField(
            model_name='collectionversion',
            name='dependencies',
            field=django.contrib.postgres.fields.jsonb.JSONField(default=dict, editable=False),
        ),
        migrations.AddField(
            model_name='collectionversion',
            name='description',
            field=models.TextField(blank=True, default='', editable=False),
        ),
        migrations.AddField(
            model_name='collectionversion',
            name='docs_blob',
            field=django.contrib.postgres.fields.jsonb.JSONField(default=dict, editable=False),
        ),
        migrations.AddField(
            model_name='collectionversion',
            name='documentation',
            field=models.URLField(blank=True, default='', editable=False, max_length=128),
        ),
        migrations.AddField(
            model_name='collectionversion',
            name='homepage',
            field=models.URLField(blank=True, default='', editable=False, max_length=128),
        ),
        migrations.AddField(
            model_name='collectionversion',
            name='issues',
            field=models.URLField(blank=True, default='', editable=False, max_length=128),
        ),
        migrations.AddField(
            model_name='collectionversion',
            name='license',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=32), default=list, editable=False, size=None),
        ),
        migrations.AddField(
            model_name='collectionversion',
            name='name',
            field=models.CharField(default='', editable=False, max_length=32),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='collectionversion',
            name='namespace',
            field=models.CharField(default='', editable=False, max_length=32),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='collectionversion',
            name='repository',
            field=models.URLField(blank=True, default='', editable=False, max_length=128),
        ),
        migrations.AlterField(
            model_name='collectionversion',
            name='version',
            field=models.CharField(editable=False, max_length=32),
        ),
        migrations.AlterUniqueTogether(
            name='collectionversion',
            unique_together={('namespace', 'name', 'version')},
        ),
        migrations.RemoveField(
            model_name='collectionversion',
            name='metadata',
        ),
        migrations.RemoveField(
            model_name='collectionversion',
            name='quality_score',
        ),
        migrations.AddField(
            model_name='collectionversion',
            name='tags',
            field=models.ManyToManyField(editable=False, to='ansible.Tag'),
        ),
    ]
