# Generated by Django 2.2.19 on 2021-03-11 23:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ansible', '0031_ansiblerepository_last_synced_metadata_time'),
    ]

    operations = [
        migrations.AddField(
            model_name='collectionremote',
            name='sync_dependencies',
            field=models.BooleanField(default=True),
        ),
    ]
